Package: mumosa
Type: Package
Authors@R: person("Aaron", "Lun", role = c("aut", "cre"), email="infinite.monkeys.with.keyboards@gmail.com")
Version: 1.19.0
Date: 2024-09-28
License: GPL-3
Title: Multi-Modal Single-Cell Analysis Methods
Description: 
    Assorted utilities for multi-modal analyses of single-cell datasets.
    Includes functions to combine multiple modalities for downstream analysis,
    perform MNN-based batch correction across multiple modalities,
    and to compute correlations between assay values for different modalities.
Depends: SingleCellExperiment
Imports: stats, utils, methods, igraph, Matrix, BiocGenerics,
        BiocParallel, IRanges, S4Vectors, DelayedArray,
        DelayedMatrixStats, SummarizedExperiment, BiocNeighbors,
        BiocSingular, ScaledMatrix, beachmat, scuttle, metapod, scran,
        batchelor, uwot
Suggests: testthat, knitr, BiocStyle, rmarkdown, scater, bluster,
        DropletUtils, scRNAseq
VignetteBuilder: knitr
biocViews: ImmunoOncology, SingleCell, RNASeq
Encoding: UTF-8
RoxygenNote: 7.3.2
URL: http://bioconductor.org/packages/mumosa
BugReports: https://support.bioconductor.org/
git_url: https://git.bioconductor.org/packages/mumosa
git_branch: devel
git_last_commit: 67c3814
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-03
NeedsCompilation: no
Packaged: 2025-11-03 23:50:07 UTC; biocbuild
Author: Aaron Lun [aut, cre]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
