# Generated by roxygen2: do not edit by hand

export(calculateMultiUMAP)
export(computeCorrelations)
export(findTopCorrelations)
export(intersectClusters)
export(intersectGraphs)
export(multiModalMNN)
export(rescaleByNeighbors)
export(runMultiUMAP)
exportMethods(calculateMultiUMAP)
exportMethods(computeCorrelations)
exportMethods(findTopCorrelations)
exportMethods(rescaleByNeighbors)
import(methods)
importFrom(BiocGenerics,cbind)
importFrom(BiocGenerics,match)
importFrom(BiocGenerics,rbind)
importFrom(BiocGenerics,unique)
importFrom(BiocNeighbors,KmknnParam)
importFrom(BiocNeighbors,buildIndex)
importFrom(BiocNeighbors,findDistance)
importFrom(BiocNeighbors,findKNN)
importFrom(BiocNeighbors,queryKNN)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bpstart)
importFrom(BiocParallel,bpstop)
importFrom(BiocSingular,IrlbaParam)
importFrom(BiocSingular,runPCA)
importFrom(DelayedArray,DelayedArray)
importFrom(DelayedArray,currentViewport)
importFrom(DelayedArray,getAutoBPPARAM)
importFrom(DelayedArray,makeNindexFromArrayViewport)
importFrom(DelayedArray,setAutoBPPARAM)
importFrom(DelayedMatrixStats,colVars)
importFrom(DelayedMatrixStats,rowAnys)
importFrom(DelayedMatrixStats,rowVars)
importFrom(IRanges,heads)
importFrom(Matrix,colMeans)
importFrom(Matrix,rowSums)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,t)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,List)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,selfmatch)
importFrom(S4Vectors,splitAsList)
importFrom(ScaledMatrix,ScaledMatrix)
importFrom(SingleCellExperiment,"reducedDim<-")
importFrom(SingleCellExperiment,altExp)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,simplifyToSCE)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,rowData)
importFrom(batchelor,applyMultiSCE)
importFrom(batchelor,convertPCsToSCE)
importFrom(batchelor,cosineNorm)
importFrom(batchelor,multiBatchPCA)
importFrom(batchelor,reducedMNN)
importFrom(beachmat,rowBlockApply)
importFrom(igraph,"E<-")
importFrom(igraph,E)
importFrom(igraph,ends)
importFrom(igraph,make_graph)
importFrom(metapod,averageParallelStats)
importFrom(metapod,parallelStouffer)
importFrom(scran,rhoToPValue)
importFrom(scran,scaledColRanks)
importFrom(scuttle,.bpNotSharedOrUp)
importFrom(scuttle,.subset2index)
importFrom(scuttle,.unpackLists)
importFrom(scuttle,normalizeCounts)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(utils,head)
importFrom(uwot,umap)
