# Multi-modal methods for single-cell analyses

|Build|Status|
|-----|----|
| Bioc-release | [![](http://bioconductor.org/shields/build/release/bioc/mumosa.svg)](http://bioconductor.org/checkResults/release/bioc-LATEST/mumosa) |
| Bioc-devel   | [![](http://bioconductor.org/shields/build/devel/bioc/mumosa.svg)](http://bioconductor.org/checkResults/devel/bioc-LATEST/mumosa) | 

This package some analysis methods for multi-modal single-cell data.
Check out the user guide on the [Bioconductor landing page](https://bioconductor.org/packages/devel/bioc/html/mumosa.html) for more details.
